#!/usr/bin/pypy-1.9

import time
import sys

def fileLen(fname):
    with open(fname) as f:
        for i, l in enumerate(f):
            pass
    return i + 1

def init(buffer):
   from glob import globs, setGlobals
   from array import array

   # create spatial variables of specified number
   temp = buffer.pop(0)
   Vars = int(temp.split()[0].strip())+2
   TypeId = temp.split()[1].strip()
   
   # initialize constraints
   N = fileLen(globs['calc']+"/"+globs['calc']+".relations")
   setGlobals("size", N)
   Id = 0
   from bitcoding import B_dict
   with open(globs['calc']+"/"+globs['calc']+".identity") as f:
      Id = B_dict[f.readline().strip()]
  
   ConMatrix = [None for v in xrange(Vars)]
   for v in xrange(Vars):
      j = {}   
      j[v] = Id
      ConMatrix[v] = j

   # parse spatial CSP
   from parsecsp import parsecsp
   parsecsp(ConMatrix, buffer)
   
   return TypeId, ConMatrix

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

# main function
def main(argv=None):

   if argv is None:
        argv = sys.argv

   # set some sys useful stuff
   sys.setrecursionlimit(1<<30)
   sys.setcheckinterval(10000)

   from optparse import OptionParser

   # initialize parser for command line arguments
   parser = OptionParser()
   
   # set parsing options
   parser.add_option("-c", "--calculus", type="string", dest="calc", default="rcc8", help="interaction mode: rcc8, or allen [default: %default]")
   parser.add_option("-d", "--print", action="store_true", dest="printsol", help="print solution to output")
   parser.add_option("-f", "--file", action="store", type="string", dest="filename")

   # parse command line arguments
   options, args = parser.parse_args()

   from glob import setGlobals
   #import argparse

   try:   
  
      # check and set globals for calculus information
      if not (options.calc == "rcc8" or options.calc == "allen"):
         raise Usage("Invalid option: " + options.calc)
      else:
         setGlobals('calc',options.calc)

   except Usage, err:
        print err.msg
        parser.print_help()
        return 1

   with open(options.filename) as f:
      total = totalcon = 0
      meter = 0
      totalMCS = 0
      buffer = []

      for i in f:
         buffer.append(i)

         if i.strip() == '.':
            start = elapsed = elapsedMCS = 0
            meter += 1

            from counters import counter

            TypeId, ConMatrix = init(buffer)
            buffer = []

            from triangulation import MCS, FIC
            from bitcoding import B_dict
               
            edjes = set([])

            neighbors = tuple([set([]) for i in xrange(len(ConMatrix))])
            for n, i in enumerate(ConMatrix):
               for j in i:
                  if j > n:
                     edjes.add((n,j))
                     neighbors[n].add(j)
                     neighbors[j].add(n)

            # variable ordering computation with MCS
            start = time.clock()
            a,a_ = MCS(ConMatrix, neighbors)
            elapsedMCS = time.clock() - start
            totalMCS += elapsedMCS

            fill = FIC(ConMatrix, neighbors, (a,a_))

            from bitcoding import B_dict
            DALL = B_dict['DALL']

            for i,j in fill:
               if (i,j) not in edjes:
                  ConMatrix[i][j] = DALL
                  ConMatrix[j][i] = DALL
                  neighbors[i].add(j)
                  neighbors[j].add(i)

            from varelimination import varElimination

            start = time.clock()
            solution = varElimination(ConMatrix,neighbors,(a,a_))
            elapsed = time.clock() - start
            total += elapsed

            # print values of complexity counters
            print '--------------------------------------'
            print 'Type ID: ', TypeId
            print '--------------------------------------'
            print '# of constraint checks: ', counter['cons']
            print '--------------------------------------'
            print 'Time for obtaining variable ordering with MCS: ', elapsedMCS, 'sec'
            print '--------------------------------------'
            print 'Time for applying directional weak path consistency: ', elapsed, 'sec'
            print '--------------------------------------'

            totalcon += counter['cons']
            counter['arcs'] = 0
            counter['cons'] = 0
            counter['nodes'] = 0
            counter['decisions'] = 0
  
            # print solution  
            if solution != None:
               print 'Consistent'
               if options.printsol == True:
                  from printm import printMatrix
                  printMatrix(solution)
            else:
               print 'Inconsistent'
            print '--------------------------------------'

            ConMatrix = None
          
   print '======================================'
   print 'SUMMARY'
   print '======================================'
   print 'Total # of constraint checks', totalcon
   print '--------------------------------------'
   print 'Total time for obtaining variable ordering with MCS:', totalMCS, 'sec'
   print '--------------------------------------'
   print 'Total time for applying directional weak path consistency:', total, 'sec'
   print '--------------------------------------'
   print 'Avg. # of constraint checks per CSP:', totalcon/(1.0*meter)
   print '--------------------------------------'
   print 'Avg. time for obtaining variable ordering with MCS per CSP:', totalMCS/(1.0*meter), 'sec'
   print '--------------------------------------'
   print 'Avg. time for applying directional weak path consistency per CSP:', total/(1.0*meter), 'sec'
   print '======================================'

if __name__ == '__main__':
   sys.exit(main())

